Option Explicit On 
Option Strict On

Imports System
Imports System.Drawing

Public Class ICards

#Region "Drawing Api/Enums"

    'Used to initiate the drawing 
    Private Declare Function cdtInit Lib "cards.dll" (ByRef width As Integer, ByRef height As Integer) As Boolean
    Private Declare Function cdtDraw Lib "cards.dll" (ByVal hdc As IntPtr, ByVal x As Integer, ByVal y As Integer, ByVal card As Integer, ByVal mode As Integer, ByVal Color As Long) As Boolean
    'used to actually draw the card
    Private Declare Function cdtDrawExt Lib "cards.dll" (ByVal hdc As IntPtr, ByVal x As Integer, ByVal y As Integer, ByVal dx As Integer, ByVal dy As Integer, ByVal card As Integer, ByVal suit As Integer, ByVal color As Long) As Boolean

    'used to close the drawing session
    Private Declare Sub cdtTerm Lib "cards.dll" ()

    Public Enum cardback
        sky = 54
        blue = 55
        fish = 56
        frog = 57
        redWhirl = 58
        beach = 59
        blueSquare = 60
        purple = 61
        desert = 62
        astronaught = 63
        orange = 64
        cars = 65
        x = 66
        o = 67
    End Enum
    'holds the names of the card face
    Public Enum Face
        Ace
        Two
        Three
        Four
        Five
        Six
        Seven
        Eight
        Nine
        Ten
        Jack
        Queen
        King
    End Enum

    'Just holds the names of the card suits
    Public Enum Suit
        Diamond
        Club
        Heart
        Spade
    End Enum

#End Region

#Region "Variables and Structures"
    'Drawing Card Variables
    Private nSuit As Suit 'Holds the suit of the currently drawn card
    Private nFace As Face 'holds the face of the currently drawn card

    Private width As Integer = 0  'This is the width of the card
    Private height As Integer = 0  'this is the height of the card

    'This structure is used to keep track of where each card is
    Private Structure CardData
        Dim index As Integer
        Dim CardType As String
        Dim Location As String
        Dim PlayerID As String
    End Structure

    'Public Variables
    Public isSet As Boolean 'used to determine if the class has bene initialised!

    'Other Private Variables
    Private PlayerCount As Integer 'Holds the number of players in the game
    Private backstyle As Integer = 61 'holds the back flip view of the card

    'Cards Variables
    Private deck(51) As String 'Holds all the cards
    Private activeDeck(51) As CardData 'Holds all the cards locations after been shuffled
    Private currentIndex As Integer 'This contains what current card number we are up to in the deck!
#End Region


    Public Sub setUp(ByVal nPlayers As Integer)
        'if we are allready set we dont need to start again
        If isSet = True Then
            Exit Sub
        End If

        Dim i As Integer 'counter
        Dim t As Integer 'counter
        Dim CardCount As Integer 'Another counter to determine what card number we are up to

        PlayerCount = nPlayers

        CardCount = 0

        'these loops fill our array with all the cards
        For t = 0 To 3
            For i = 0 To 12
                Select Case t
                    Case 0
                        deck(CardCount) = "h." & i + 1
                    Case 1
                        deck(CardCount) = "d." & i + 1
                    Case 2
                        deck(CardCount) = "s." & i + 1
                    Case 3
                        deck(CardCount) = "c." & i + 1
                End Select
                CardCount = CardCount + 1
            Next
        Next

        'Now we must set up the drawing section
        cdtInit(width, height)
        Application.DoEvents()

        isSet = True ' set the variable to true so we can tell that the class has been initialised!
    End Sub

    Public Sub Shuffle(ByVal nTimes As Integer)
        'this sub shuffles the deck and moves the values to activeDeck

        Dim x As Integer
        Dim swap As String
        Dim r As Random = New Random

        For p As Integer = 0 To nTimes
            For i As Integer = 0 To deck.GetUpperBound(0)
                x = r.Next(0, i)

                swap = deck(x)
                deck(x) = deck(i)
                deck(i) = swap
            Next i
        Next p

        For i As Integer = 0 To 51
            activeDeck(i).index = i
            activeDeck(i).CardType = deck(i)
            activeDeck(i).Location = "."
            activeDeck(i).PlayerID = "."
        Next

    End Sub

    Public Sub collect(ByVal nPlayer As Integer)
        'this sub collects cards in from the designated player id and sets location values to .
        For i As Integer = 0 To 51
            If activeDeck(i).PlayerID = CType(nPlayer, String) Then
                activeDeck(i).Location = "."
                activeDeck(i).PlayerID = "."
            End If
        Next
    End Sub

    Public Function Deal(ByVal nPlayer As Integer) As String
        'this sub deals a card to a nominated player and stores the player id in the active deck array
        Dim i As Integer = currentIndex

        If activeDeck(i).Location = "." Then
            If currentIndex >= 51 Then
                currentIndex = 0
            Else
                currentIndex = currentIndex + 1
            End If
            activeDeck(i).Location = "+"
            activeDeck(i).PlayerID = CType(nPlayer, String)
            Return activeDeck(i).CardType
        End If
    End Function

    Public Function returnCards(ByVal nplayer As Integer) As String
        Dim temp As String

        For i As Integer = 0 To 51
            If activeDeck(i).PlayerID = CType(nplayer, String) Then
                temp = temp & "," & activeDeck(i).CardType
            End If
        Next

        Return temp
    End Function

    Public Sub DrawCard(ByVal nCard As String, ByVal g As Graphics, ByVal x As Integer, ByVal y As Integer)
        Dim number As String 'will hold our face from our splitted string
        Dim isuit As String 'will hold our suit from our splited string
        Dim hdc As IntPtr = g.GetHdc() 'stores out gfx location

        'split the string will be in the format ("Suit.Face")
        Dim temp() As String = Split(nCard, ".")

        isuit = temp(0)
        number = temp(1)

        'calls 2 routines to transform the suit/face to correct words
        nSuit = convertSuit(isuit)
        nFace = convertFace(number)

        'draw our card and release the memory
        Try
            Dim Card As Integer = CType(Me.nFace, Integer) * 4 + nSuit
            cdtDrawExt(hdc, x, y, MyClass.width, MyClass.height, Card, 0, 0)
        Finally
            g.ReleaseHdc(hdc)
        End Try
    End Sub

    Public Sub drawBack(ByVal style As cardback, ByVal g As Graphics, ByVal x As Integer, ByVal y As Integer)
        Dim hdc As IntPtr = g.GetHdc()
        Try
            cdtDraw(hdc, x, y, style, 1, 0)
        Finally
            g.ReleaseHdc(hdc)
        End Try
    End Sub

    Public Sub Close()
        cdtTerm()
    End Sub

    Private Function convertFace(ByVal iFace As String) As Face
        Select Case iFace
            Case "1"
                Return Face.Ace
            Case "2"
                Return Face.Two
            Case "3"
                Return Face.Three
            Case "4"
                Return Face.Four
            Case "5"
                Return Face.Five
            Case "6"
                Return Face.Six
            Case "7"
                Return Face.Seven
            Case "8"
                Return Face.Eight
            Case "9"
                Return Face.Nine
            Case "10"
                Return Face.Ten
            Case "11"
                Return Face.Jack
            Case "12"
                Return Face.Queen
            Case "13"
                Return Face.King
        End Select
    End Function

    Private Function convertSuit(ByVal iSuit As String) As Suit
        Select Case iSuit
            Case "h"
                Return Suit.Heart
            Case "d"
                Return Suit.Club
            Case "s"
                Return Suit.Spade
            Case "c"
                Return Suit.Diamond
        End Select
    End Function


End Class
